uniform float   Time;
uniform float   CubeScale; // the scale of the cube below us
uniform float   Spin;

const vec3      testPosition    = vec3( 0.0, 0.0, 40.0 );
const vec3      testNormal      = vec3( 0.0, 1.0, 0.0 );

// parameters for the wave function
const float     xdiv = 10.0;
const float     zdiv = 8.0;
const float     magx = 1.6;
const float     magz = 1.0;
mat4      identity = mat4 (   1.0, 0.0, 0.0, 0.0,
                              0.0, 1.0, 0.0, 0.0,
                              0.0, 0.0, 1.0, 0.0,
                              0.0, 0.0, 0.0, 1.0);

varying vec3    normal;

vec3            tn;
vec3            tp;
mat4            mvmat; // to be used instead of modelview matrix
mat3            nmat; // to be used instead of normal matrix
mat4            tempmat;
mat4            wave_rot_mat;
//-----------------------------------------------------------------
void wave_func(void); 
void build_norm_mat(void); 
//-----------------------------------------------------------------
void main(void)
{
    tp = vec3(0.0,0.0,0.0);
    tn = vec3(0.0,0.0,0.0);
    mvmat = gl_ModelViewMatrix;
    nmat = gl_NormalMatrix; 
   
    // apply wave displacement
    wave_func();

    // translation & scale
    tempmat = identity;
    tempmat[3][0] = 0.0;
    tempmat[3][1] = tp.y;
    tempmat[3][2] = testPosition.z; 
    mvmat *= tempmat;
 
    // apply rotation due to normal at wave
    mvmat *= wave_rot_mat;

    // apply rotation about y-axis  and translate in y to place above cube
    tempmat = identity;
    tempmat[0][0] = cos(Spin);
    tempmat[2][0] = sin(Spin);
    tempmat[0][2] = -sin(Spin);
    tempmat[2][2] = cos(Spin);
    tempmat[3][1] = CubeScale;
    mvmat *= tempmat;
    
    // rebuild normal matrix
    build_norm_mat();
   
    // we happen to know the scale of the cube below us, let's reuse tp
    tp = gl_Vertex.xyz ; 
    normal = normalize( nmat * gl_Normal );
    gl_Position = gl_ProjectionMatrix * mvmat * vec4(tp,1);
}

//-----------------------------------------------------------------
float determinant(mat3 m)
{
    float t = 0.0;
    t = m[0][0] * ((m[1][1] * m[2][2]) - (m[2][1] * m[1][2]));
    t+= m[0][1] * ((m[1][2] * m[2][0]) - (m[2][2] * m[1][0]));
    t+= m[0][2] * ((m[1][0] * m[2][1]) - (m[2][0] * m[1][1]));

    return t;
}

float determinant(mat2 m)
{
    return ((m[0][0] * m[1][1]) - (m[1][0] * m[0][1]));
}

//-----------------------------------------------------------------
// normal matrix is transpose of inverse of upper left 3x3 of modelview matrix
void build_norm_mat(void) 
{
    mat3  tnmat;
    float d;
    for( int i = 0; i < 3; i++)
        for( int k = 0; k < 3; k++)
            nmat[i][k] = mvmat[i][k];
    d = determinant(nmat);
    
    // take determinant and transpose at same time
    tnmat[0][0] = 
        determinant(mat2(nmat[1][1], nmat[1][2], nmat[2][1], nmat[2][2]));
    tnmat[1][0] = 
        determinant(mat2(nmat[0][2], nmat[0][1], nmat[2][2], nmat[2][1]));
    tnmat[2][0] = 
        determinant(mat2(nmat[0][1], nmat[0][2], nmat[1][1], nmat[1][2]));
    tnmat[0][1] = 
        determinant(mat2(nmat[1][2], nmat[1][0], nmat[2][2], nmat[2][1]));
    tnmat[1][1] = 
        determinant(mat2(nmat[0][0], nmat[0][2], nmat[2][1], nmat[2][2]));
    tnmat[2][1] = 
        determinant(mat2(nmat[1][2], nmat[0][0], nmat[1][2], nmat[1][0]));
    tnmat[0][2] = 
        determinant(mat2(nmat[1][0], nmat[1][1], nmat[2][0], nmat[2][1]));
    tnmat[1][2] = 
        determinant(mat2(nmat[0][1], nmat[0][0], nmat[2][1], nmat[2][0]));
    tnmat[2][2] = 
        determinant(mat2(nmat[0][0], nmat[0][1], nmat[1][0], nmat[1][1]));

    nmat = tnmat / d;
}

//-----------------------------------------------------------------
void wave_func(void) 
{
    vec3 v1, v2, pos1, pos2;
    float h1, h2;
    
    // determine current height displacement 
    h1 = sin((testPosition.z / zdiv) + Time) * magz; 
    h2 = sin((testPosition.x / xdiv) + Time) * magx +
         sin( (testPosition.x / 5.0 ) + (Time - 4.0));; 
    tp = vec3( 0.0, h1 + h2, 0.0); // only take y component
    
    // get our two slopes in xy and zy
    // d/dx( sin(u) ) = cos(u)d/dx(u)
    // v1 with respect to z
    // v2 with respect to x
    v1 = vec3(  0.0, 
                cos((testPosition.z / zdiv) + Time) * magz / zdiv, 
                1.0);    
    v2 = vec3(  1.0, 
                cos((testPosition.x / xdiv) + Time) * magx / xdiv +
                cos( (testPosition.x / 5.0 ) + (Time - 4)) / 5.0, 
                0.0);   
    v1 = normalize(v1); 
    v2 = normalize(v2);

    // find our normal, should be in positive y direction
    tn = normalize(cross(v1,v2));

    // should have our basis vectors!! v1, v2, tn... now build a matrix
    wave_rot_mat = identity;
    wave_rot_mat[0] = vec4( v2, 0.0);
    wave_rot_mat[1] = vec4( tn, 0.0);
    wave_rot_mat[2] = vec4( v1, 0.0);
}

